#!/bin/sh
exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}
 if 0;
#!/usr/bin/perl -w

# Previously the admin controller and the web server controller had
# separate sslwiz.cgi programs.  These were maintained almost but not
# quite in sync, but no longer.
#
# Now we have just one instance of sslwiz.cgi, and it's the one under
# the admin controller directory.  There are still two versions of
# sslwiz.ahtml, however.
#
# sslwiz.cgi expects the "server" cgi parameter to be set to the name
# of the website to secure, or to "admin website" if it is the admin
# website to be secured.

BEGIN { @INC=( "$ENV{ZEUSHOME}/perl/", "$ENV{ZEUSHOME}/admin/etc/" ); }
use Z;

# First off, ensure this page isn't cached, otherwise it can be very
# confusing!
$| = 1;

print <<EOM;
Pragma: no-cache\r
Cache-Control: no-cache\r
Expires: 0\r
Content-Type: text/html\r
\r
<body bgcolor=white>
EOM

Z::set_settings(
  "base"	=> ""
);

# We're expecting the following args:
#   back
#   !security!private_key, !security!public_key,
#   key_size, cert_request, dns, org, orgu, loc, state, country, 
#   verbose, clobber

$private_key  = $::f{"!security!private_key"}  
                                   || FATAL("No private key file specified");
$key_size     = $::f{key_size}     || "1024";
$public_cert  = $::f{"!security!public_key"} || ""; 
$cert_request = $::f{cert_request} || "";

if( ! ( $public_cert or $cert_request )) {
    FATAL( "You must specify at least one of a certificate request file or a self-signed public certificate file." );
}

$id_dns       = $::f{dns}     || FATAL("No 'dns' value supplied");
$id_org       = $::f{org}     || FATAL("No 'Organisation' value supplied");
$id_orgu      = $::f{orgu}    || ".";
$id_loc       = $::f{loc}     || FATAL("No 'location' value supplied");
$id_state     = $::f{state}   || ".";
$id_country   = $::f{country} || FATAL("No 'country' value supplied");

$verbose      = $::f{verbose} || "";
$clobber      = $::f{clobber} || "";

$private_key =~ /^\// or 
   FATAL("private key must be an absolute file name (beginning '/')");
( $cert_request and $cert_request !~ /^\// ) and
   FATAL("certificate request must be an absolute file name (beginning '/')");
( $public_cert and $public_cert !~ /^\// ) and
   FATAL("public certificate must be an absolute file name (beginning '/')");


print "<dl>\n";

print "<dt><h3>Generating private key</h3>\n<dd>\n";

if( -f $private_key ) {
   $/ = "\n";
   open KEY, "<$private_key";
   if (<KEY> !~ /^-----BEGIN RSA PRIVATE KEY-----\n$/) {
      FATAL("$private_key exists and is not a valid private key\n");
   }
   close KEY;
   print "<b>$private_key</b> already exists; using this file\n";
} else {
   Z::verify_output_file("Private key file", $private_key);
   $cmd = "\$ZEUSHOME/admin/bin/cert -new -type private -keysize $key_size -out $private_key";
   
   $verbose and print "<pre><b>$cmd</b>\n\n";

   $/ = "\n";
   open CMD, "$cmd 2>&1 |";
   while (<CMD>) { $verbose and print "   $_"; }
   $verbose and print "</pre>\n";

   close CMD;
   $? and FATAL("</dl>Command exited abnormally - aborting\n");
   
   print "Your new private key has been written to '$private_key'\n";
}

if( $cert_request ) {
   print "<dt><h3>Generating certificate request</h3>\n<dd>\n";

   if( !$clobber and -f $cert_request ) {
      FATAL("$cert_request exists; please move or delete this file and try again\n");
   } else {
      Z::verify_output_file("Certificate request file", $cert_request);
      $cmd = "\$ZEUSHOME/admin/bin/cert -new -type request -key $private_key -out $cert_request";
      
      $verbose and print "<pre><b>$cmd</b>\n";
      ( $exit, $stdout, $stderr ) = Z::run_external($cmd, "$id_country\n$id_state\n$id_loc\n$id_org\n$id_orgu\n$id_dns\n" );

      $exit and FATAL("</dl>Command exited abnormally - aborting\n");

      $txt = "   ".$stdout.$stderr;

      $txt =~ s/:  /: <b>$id_country<\/b>\n/;
      $txt =~ s/:  /: <b>$id_state<\/b>\n/;
      $txt =~ s/:  /: <b>$id_loc<\/b>\n/;
      $txt =~ s/:  /: <b>$id_org<\/b>\n/;
      $txt =~ s/:  /: <b>$id_orgu<\/b>\n/;
      $txt =~ s/:  /: <b>$id_dns<\/b>\n/;
      $txt =~ s/\n/\n   /g; #indent
      $verbose and print "$txt</pre>\n";

      print "Your certificate request has been written to the file '$cert_request'.\n";
   }

   if ($verbose||1) {
      print "<dt><h3>Displaying certificate request</h3>\n<dd>\n";

      $cmd = "cat $cert_request";
      
      print "<pre><b>$cmd</b>\n</pre></dl><xmp>";
      
      $/ = "\n";
      open CMD, "$cmd 2>&1 |";
      while (<CMD>) { print "$_"; }
      print "</xmp><dl>\n";
      close CMD;
      $? and FATAL("</dl>Command exited abnormally - aborting\n");
   }
}

if( $public_cert ) {
   print "<dt><h3>Generating public certificate</h3>\n<dd>\n";
   
   if( !$clobber and -f $public_cert ) {
      FATAL("$public_cert exists; please move or delete this file and try again\n");
   } else {
      Z::verify_output_file("Public certificate file", $public_cert);
      $cmd = "\$ZEUSHOME/admin/bin/cert -new -type public -key $private_key -out $public_cert";
      
      $verbose and print "<pre><b>$cmd</b>\n";
      ( $exit, $stdout, $stderr ) = Z::run_external($cmd,
                        "$id_country\n$id_state\n$id_loc\n$id_org\n$id_orgu\n$id_dns\n");

      $exit and FATAL("</dl>Command exited abnormally - aborting\n");
   
      $txt = "   ".$stdout.$stderr;
      
      $txt =~ s/:  /: <b>$id_country<\/b>\n/;
      $txt =~ s/:  /: <b>$id_state<\/b>\n/;
      $txt =~ s/:  /: <b>$id_loc<\/b>\n/;
      $txt =~ s/:  /: <b>$id_org<\/b>\n/;
      $txt =~ s/:  /: <b>$id_orgu<\/b>\n/;
      $txt =~ s/:  /: <b>$id_dns<\/b>\n/;
      $txt =~ s/\n/\n   /g; #indent
      $verbose and print "$txt</pre>\n";
   
      print "Your new public certificate has been written to the file '$public_cert'.\n";
   }
   
   if( $verbose ) {
      print "<dt><h3>Displaying public certificate</h3>\n<dd>\n";
   
      $cmd = "\$ZEUSHOME/admin/bin/cert -in $public_cert -text";
      
      print "<pre><b>$cmd</b>\n\n";
      
      $/ = "\n";
      open CMD, "$cmd 2>&1 |";
      
      while (<CMD>) { print "   $_"; }
      print "</pre>\n";
      
      close CMD;
      $? and FATAL("</dl>Command exited abnormally - aborting\n");
   }
   
   print "<dt><h3>Verifying Key/Certificate pair</h3>\n<dd>\n";
   
   $cmd = "\$ZEUSHOME/admin/bin/cert -check -key $private_key -in $public_cert";
   
   $verbose and print "<pre><b>$cmd</b>\n\n";
   
   $/ = "\n";
   open CMD, "$cmd 2>&1 |";
   
   while (<CMD>) { $verbose and print "   $_"; }
   $verbose and print "</pre>\n";
   
   close CMD;
   $? and FATAL("</dl>Command exited abnormally - aborting\n");
   
   print "Done\n";
}

# Commit changes.  If we've got this far, all is OK

print "<dt><h3>Updating Website</h3>\n<dd>\n";

my( $zc ) = new ZConf;

# pick the correct config file either for a "real" website or for
# the admin website
if(defined $::f{server}) {
   $configfile = "$ENV{ZEUSHOME}/webadmin/conf/sites/$::f{server}";
} else {
   $configfile = "$ENV{ZEUSHOME}/admin/website";
}

$zc->ZConf::load( $configfile );

foreach $key (grep /^\!/, sort keys %::f) {
   ($keyf = $::f{base}.$key) =~ s/^\!//;
   $zc->ZConf::add($keyf, $::f{$key});
}

$zc->ZConf::add( "security!enabled", "yes" );

$zc->ZConf::save();

$verbose and print
    "  <p>Saved configuration to $configfile.  You must\n",
    "  restart the website before your changes will take effect.</p>\n";

print "Done</dl>\n";

print footer();
exit 0;

sub FATAL {
   local $msg = shift;
   print "<b>Error</b>: $msg\n";
   print footer();
   exit 0;
}

# callback from Z.pm
sub Error {
   FATAL(@_);
}

sub footer {
   if( $::f{back}) {
      return "<p><a href=\"$::f{back}\"><img src=\"/apps/admin/back.gif\" border=0></a>\n";
   } else {
      return "\n";
   }
}

# Local Variables:
# mode:perl
# perl-indent-level:3
# End:
